import h5py
import numpy as np
import warnings

warnings.filterwarnings("ignore")


__version__ = '0.9.0'


def abspath(path):
    r'''
Return absolute path.
    '''

    import posixpath

    return posixpath.normpath(posixpath.join('/', path))


def join(*args, root=False):
    r'''
Join path components.
    '''

    import posixpath

    lst = []

    for i, arg in enumerate(args):
        if i == 0:
            lst += [arg]
        else:
            lst += [arg.strip('/')]

    if root:
        return posixpath.join('/', *lst)

    return posixpath.join(*lst)


def getdatasets(data, root='/'):
    r'''
Iterator to transverse all datasets across all groups in HDF5 file. Usage:

.. code-block:: python

    with h5py.File('...', 'r') as data:

        # loop over all paths
        for path in GooseHDF5.getdatasets(data):
            print(path)

        # get a set of all datasets
        paths = set(GooseHDF5.getdatasets(data))

        # get a list of all datasets
        paths = list(GooseHDF5.getdatasets(data))

Read more in `this answer <https://stackoverflow.com/a/50720736/2646505>`_.

:arguments:

    **data** (``<h5py.File>``)
        A HDF5-archive.

:options:

    **root** ([``'/'``] | ``<str>``)
        Start a certain point along the path-tree.

:returns:

    Iterator.
    '''

    # ---------------------------------------------

    def iterator(g, prefix):

        for key in g.keys():

            item = g[key]
            path = join(prefix, key)

            if isinstance(item, h5py.Dataset):
                yield path

            elif isinstance(item, h5py.Group):
                yield from iterator(item, path)

    # ---------------------------------------------

    if isinstance(data[root], h5py.Dataset):
        yield root

    for path in iterator(data[root], root):
        yield path


def getpaths(data, root='/', max_depth=None, fold=None):
    r'''
Iterator to transverse all datasets across all groups in HDF5 file. Whereby one can choose to fold
(not transverse deeper than):

- Groups deeper than a certain ``max_depth``.
- A (list of) specific group(s).

:arguments:

    **data** (``<h5py.File>``)
        A HDF5-archive.

:options:

    **root** ([``'/'``] | ``<str>``)
        Start a certain point along the path-tree.

    **max_depth** (``<int>``)
        Set a maximum depth beyond which groups are folded.

    **fold** (``<list>``)
        Specify groups that are folded.

:returns:

    Iterator.

:example:

    Consider this file:

    .. code-block:: bash

        /path/to/first/a
        /path/to/first/b
        /data/c
        /data/d
        /e

    Calling:

    .. code-block:: python

        with h5py.File('...', 'r') as data:

            for path in GooseHDF5.getpaths(data, max_depth=2, fold='/data'):
                print(path)

    Will print:

    .. code-block:: bash

        /path/to/...
        /data/...
        /e

    The ``...`` indicate that it concerns a folded group, not a dataset.
    Here, the first group was folded because of the maximum depth, and the second because it was
    specifically requested to be folded.
    '''

    if max_depth and fold:
        return _getpaths_fold_maxdepth(data, root, fold, max_depth)

    if max_depth:
        return _getpaths_maxdepth(data, root, max_depth)

    if fold:
        return _getpaths_fold(data, root, fold)

    return _getpaths(data, root)


def _getpaths(data, root):
    r'''
Specialization for ``getpaths``
    '''

    # ---------------------------------------------

    def iterator(g, prefix):

        for key in g.keys():

            item = g[key]
            path = join(prefix, key)

            if isinstance(item, h5py.Dataset):
                yield path

            elif isinstance(item, h5py.Group):
                yield from iterator(item, path)

    # ---------------------------------------------

    if isinstance(data[root], h5py.Dataset):
        yield root

    for path in iterator(data[root], root):
        yield path


def _getpaths_maxdepth(data, root, max_depth):
    r'''
Specialization for ``getpaths`` such that:

- Groups deeper than a certain maximum depth are folded.
    '''

    # ---------------------------------------------

    def iterator(g, prefix, max_depth):

        for key in g.keys():

            item = g[key]
            path = join(prefix, key)

            if isinstance(item, h5py.Dataset):
                yield path

            elif len(path.split('/')) - 1 >= max_depth:
                yield path + '/...'

            elif isinstance(item, h5py.Group):
                yield from iterator(item, path, max_depth)

    # ---------------------------------------------

    if isinstance(max_depth, str):
        max_depth = int(max_depth)

    if isinstance(data[root], h5py.Dataset):
        yield root

    for path in iterator(data[root], root, max_depth):
        yield path


def _getpaths_fold(data, root, fold):
    r'''
Specialization for ``getpaths`` such that:

- Certain groups are folded.
    '''

    # ---------------------------------------------

    def iterator(g, prefix, fold):

        for key in g.keys():

            item = g[key]
            path = join(prefix, key)

            if isinstance(item, h5py.Dataset):
                yield path

            elif path in fold:
                yield path + '/...'

            elif isinstance(item, h5py.Group):
                yield from iterator(item, path, fold)

    # ---------------------------------------------

    if isinstance(fold, str):
        fold = [fold]

    if isinstance(data[root], h5py.Dataset):
        yield root

    for path in iterator(data[root], root, fold):
        yield path


def _getpaths_fold_maxdepth(data, root, fold, max_depth):
    r'''
Specialization for ``getpaths`` such that:

- Certain groups are folded.
- Groups deeper than a certain maximum depth are folded.
    '''

    # ---------------------------------------------

    def iterator(g, prefix, fold, max_depth):

        for key in g.keys():

            item = g[key]
            path = join(prefix, key)

            if isinstance(item, h5py.Dataset):
                yield path

            elif len(path.split('/')) - 1 >= max_depth:
                yield path + '/...'

            elif path in fold:
                yield path + '/...'

            elif isinstance(item, h5py.Group):
                yield from iterator(item, path, fold, max_depth)

    # ---------------------------------------------

    if isinstance(max_depth, str):
        max_depth = int(max_depth)

    if isinstance(fold, str):
        fold = [fold]

    if isinstance(data[root], h5py.Dataset):
        yield root

    for path in iterator(data[root], root, fold, max_depth):
        yield path


def filter_datasets(data, paths):
    r'''
From a list of paths filter those paths that do not point to datasets.

This function can for example be used in conjunction with "getpaths":

.. code-block:: python

    with h5py.File('...', 'r') as data:

        datasets = GooseHDF5.filter_datasets(data,
            GooseHDF5.getpaths(data, max_depth=2, fold='/data'))

:arguments:

    **data** (``<h5py.File>``)
        A HDF5-archive.

    **paths** (``<list<str>>``)
        A list of paths to datasets.
    '''

    import re

    paths = list(paths)
    paths = [path for path in paths if not re.match(r'(.*)(/\.\.\.)', path)]
    paths = [path for path in paths if isinstance(data[path], h5py.Dataset)]
    return paths


def verify(data, datasets, error=False):
    r'''
Try reading each dataset of a list of datasets. Return a list with only those datasets that can be
successfully opened.

:arguments:

    **data** (``<h5py.File>``)
        A HDF5-archive.

    **datasets** (``<list<str>>``)
        A list of paths to datasets.

:option:

    **error** ([``False``] | ``True``)
        If true, the function raises an error if reading failed. If false, the function just
        continues.
    '''

    out = []

    for path in datasets:

        try:
            tmp = data[path][...]
        except:
            if error:
                raise IOError('Error reading "{path:s}"'.format(path=path))
            else:
                continue

        out += [path]

    return out


def exists(data, path):
    r'''
Check if a path exists in the HDF5-archive.

:arguments:

    **data** (``<h5py.File>``)
        A HDF5-archive.

    **path** (``<str>``)
        A path to datasets.
    '''

    if path in data:
        return True

    return False


def exists_any(data, paths):
    r'''
Check if any of the input paths exists in the HDF5-archive.

:arguments:

    **data** (``<h5py.File>``)
        A HDF5-archive.

    **paths** (``<list<str>>``)
        A list of paths to datasets.
    '''

    if isinstance(paths, str):
        paths = [paths]

    for path in paths:
        if exists(data, path):
            return True

    return False


def exists_all(data, paths):
    r'''
Check if all of the input paths exists in the HDF5-archive.

:arguments:

    **data** (``<h5py.File>``)
        A HDF5-archive.

    **paths** (``<list<str>>``)
        A list of paths to datasets.
    '''

    if isinstance(paths, str):
        paths = [paths]

    for path in paths:
        if not exists(data, path):
            return False

    return True


def copydatasets(source, dest, source_datasets, dest_datasets=None, root=None):
    r'''
Copy all datasets from one HDF5-archive 'source' to another HDF5-archive 'dest'. The datasets
can be renamed by specifying a list of 'dest_datasets' (whose entries should correspond to the
'source_datasets').

In addition a 'root' (path prefix) for the destination datasets name can be specified.

:arguments:

    **source, dest** (``<h5py.File>``)
        The source and destination HDF5-archives.

    **source_datatsets** (``<list<str>>``)
        A list of paths to datasets in "source".

:options:

    **dest_datasets**  (``<list<str>>``)
        A list of paths to datasets in "dest".
        If not specified, it is taken equal to "source_datasets".

    **root** (``<str>``)
        Path prefix for all 'dest_datasets'.
    '''

    import posixpath

    source_datasets = [abspath(path) for path in source_datasets]

    if not dest_datasets:
        dest_datasets = [path for path in source_datasets]

    if root:
        dest_datasets = [join(root, path, root=True) for path in dest_datasets]

    for dest_path in dest_datasets:
        if exists(dest, dest_path):
            raise IOError('Dataset "{0:s}" already exists'.format(dest_path))

    # extract groups and sort based on depth
    groups = list(set([posixpath.split(path)[0] for path in dest_datasets]))
    groups = [group for group in groups if group != '/']
    groups = sorted(groups, key=lambda group: (group.count('/'), group))

    # create groups
    for group in groups:
        if not exists(dest, group):
            dest.create_group(group)

    # copy datasets
    for source_path, dest_path in zip(source_datasets, dest_datasets):
        group = posixpath.split(dest_path)[0]
        source.copy(source_path, dest[group], posixpath.split(dest_path)[1])


def isnumeric(a):
    r'''
Returns ``True`` is an array contains numeric values.
    '''

    import numpy as np

    if type(a) == str:
        return False

    if np.issubdtype(a.dtype, np.number):
        return True

    return False


def _equal_value(a, b):

    import numpy as np

    if type(a) == str:
        if type(b) == str:
            return a == b
        else:
            return False

    if np.issubdtype(a.dtype, np.number) and np.issubdtype(b.dtype, np.number):
        if np.allclose(a, b):
            return True
        else:
            return False

    if a.size != b.size:
        return False

    if a.size == 1:
        if a[...] == b[...]:
            return True
        else:
            return False

    return list(a) == list(b)


def _equal(a, b):

    if isinstance(a, h5py.Group) and isinstance(b, h5py.Group):
        return True

    if not isinstance(a, h5py.Dataset) or not isinstance(b, h5py.Dataset):
        raise IOError('Not a Dataset')

    for key in a.attrs:
        if key not in b.attrs:
            return False
        if not _equal_value(a.attrs[key], b.attrs[key]):
            return False

    for key in b.attrs:
        if key not in a.attrs:
            return False

    return _equal_value(a, b)


def equal(source, dest, source_dataset, dest_dataset=None):
    r'''
Check that a dataset is equal in both files.

:arguments:

    **source, dest** (``<h5py.File>``)
        The source and destination HDF5-archives.

    **source_datatset** (``<str>``)
        The path to a dataset in ``source``.

:options:

    **dest_dataset**  (``<str>``)
        The path to a dataset in ``dest``.
        If not specified, it is taken equal to ``source_dataset``.
    '''

    if not dest_dataset:
        dest_dataset = source_dataset

    if source_dataset not in source:
        raise IOError('"{0:s} not in {1:s}'.format(source_dataset, source.filename))

    if dest_dataset not in dest:
        raise IOError('"{0:s} not in {1:s}'.format(dest_dataset, dest.filename))

    return _equal(source[source_dataset], dest[dest_dataset])


def allequal(source, dest, source_datasets, dest_datasets=None):
    r'''
Check that all listed datasets are equal in both files.

:arguments:

    **source, dest** (``<h5py.File>``)
        The source and destination HDF5-archives.

    **source_datatsets** (``<list<str>>``)
        A list of paths to datasets in "source".

:options:

    **dest_datasets**  (``<list<str>>``)
        A list of paths to datasets in "dest".
        If not specified, it is taken equal to "source_datasets".
    '''

    if not dest_datasets:
        dest_datasets = [path for path in source_datasets]

    for source_dataset, dest_dataset in zip(source_datasets, dest_datasets):
        if not equal(source, dest, source_dataset, dest_dataset):
            return False

    return True


def copy_dataset(source, dest, paths, compress=False, double_to_float=False):
    r'''
Copy a dataset from one file to another. This function also copies possible attributes.

:arguments:

    **source, dest** (``<h5py.File>``)
        The source and destination HDF5-archives.

    **paths** (``<str>``, ``<list<str>>``)
        The path(s) to copy.

:options:

    **compress** ([``False``] | ``True``)
        If ``True`` compression is applied to the destination dataset(s).

    **double_to_float** ([``False``] | ``True``)
        If ``True`` doubles are converted to floats before copying the dataset(s).
    '''

    if type(paths) != list:
        paths = list(paths)

    for path in paths:

        data = source[path][...]

        if data.size == 1 or not compress or not isnumeric(data):
            dest[path] = source[path][...]
        else:
            dtype = source[path].dtype
            if dtype == np.float64 and double_to_float:
                dtype = np.float32
            dset = dest.create_dataset(path, data.shape, dtype=dtype, compression="gzip")
            dset[:] = data

        for key in source[path].attrs:
            dest[path].attrs[key] = source[path].attrs[key]



'''G5compare
  Compare two HDF5 files. If the function does not output anything all datasets are present in both
  files, and all the content of the datasets is equal.
  Each output line corresponds to a mismatch between the files.

Usage:
  G5compare [options] [--renamed ARG]... <source> <other>

Arguments:
  <source>    HDF5-file.
  <other>     HDF5-file.

Options:
  -t, --dtype           Verify that the type of the datasets match.
  -r, --renamed=ARG     Renamed paths, separated by a separator (see below).
  -s, --ifs=ARG         Separator used to separate renamed fields. [default: :]
  -h, --help            Show help.
      --version         Show version.

(c - MIT) T.W.J. de Geus | tom@geus.me | www.geus.me | github.com/tdegeus/GooseHDF5
'''

from .. import equal
from .. import getpaths
from .. import __version__
import docopt
import h5py
import os
import sys
import numpy as np
import warnings
warnings.filterwarnings("ignore")


def check_isfile(fname):
    if not os.path.isfile(fname):
        raise IOError('"{0:s}" does not exist'.format(fname))


def check_dataset(source, dest, source_dataset, dest_dataset, check_dtype):
    r'''
Check if the datasets (read outside) "a" and "b" are equal. If not print a message with the "path"
to the screen and return "False".
    '''

    if not equal(source, dest, source_dataset, dest_dataset):
        print(' !=  {0:s}'.format(source_dataset))
        return False

    if check_dtype:
        if source[source_dataset].dtype != dest[dest_dataset].dtype:
            print('type {0:s}'.format(source_dataset))
            return False

    return True


def _check_plain(source, other, check_dtype):
    r'''
Support function for "check_plain."
    '''

    for path in getpaths(source):
        if path not in other:
            print('-> {0:s}'.format(path))

    for path in getpaths(other):
        if path not in source:
            print('<- {0:s}'.format(path))

    for path in getpaths(source):
        if path in other:
            check_dataset(source, other, path, path, check_dtype)


def check_plain(source_name, other_name, check_dtype):
    r'''
Check all datasets (without allowing for renamed datasets).
    '''
    with h5py.File(source_name, 'r') as source:
        with h5py.File(other_name, 'r') as other:
            _check_plain(source, other, check_dtype)


def _check_renamed(source, other, renamed, check_dtype):
    r'''
Support function for "check_renamed."
    '''

    s2o = {i: i for i in list(getpaths(source))}
    o2s = {i: i for i in list(getpaths(other))}

    for s, o in renamed:
        s2o[s] = o
        o2s[o] = s

    for _, path in s2o.items():
        if path not in o2s:
            print(' ->  {0:s}'.format(path))

    for _, path in o2s.items():
        if path not in s2o:
            print(' <-  {0:s}'.format(path))

    for new_path, path in s2o.items():
        if new_path in o2s:
            check_dataset(source, other, path, new_path, check_dtype)


def check_renamed(source_name, other_name, renamed, check_dtype):
    r'''
Check all datasets while allowing for renamed datasets.
renamed = [['source_name1', 'other_name1'], ['source_name2', 'other_name2'], ...]
    '''

    with h5py.File(source_name, 'r') as source:
        with h5py.File(other_name, 'r') as other:
            _check_renamed(source, other, renamed, check_dtype)


def main():
    r'''
Main function.
    '''

    args = docopt.docopt(__doc__, version=__version__)

    check_isfile(args['<source>'])
    check_isfile(args['<other>'])

    if len(args['--renamed']) == 0:
        check_plain(args['<source>'], args['<other>'], args['--dtype'])
        sys.exit(0)

    renamed = [i.split(args['--ifs']) for i in args['--renamed']]

    check_renamed(args['<source>'], args['<other>'], renamed, args['--dtype'])



'''G5repair
    Extract readable data from a HDF5-file and copy it to a new HDF5-file.

Usage:
    G5repair [options] <source> <destination>

Arguments:
    <source>        Source HDF5-file, possibly containing corrupted data.
    <destination>   Destination HDF5-file.

Options:
    -f, --force     Force continuation, overwrite existing files.
    -h, --help      Show help.
        --version   Show version.

(c - MIT) T.W.J. de Geus | tom@geus.me | www.geus.me | github.com/tdegeus/GooseHDF5
'''

from .. import getdatasets
from .. import copydatasets
from .. import verify
from .. import __version__
import docopt
import h5py
import os
import sys
import warnings
warnings.filterwarnings("ignore")


def check_isfile(fname):
    if not os.path.isfile(fname):
        raise IOError('"{0:s}" does not exist'.format(fname))


def main():

    args = docopt.docopt(__doc__, version=__version__)

    check_isfile(args['<source>'])

    if os.path.isfile(args['<destination>']) and not args['--force']:
        if not click.confirm('File "{0:s}" exists, continue [y/n]? '.format(args['<destination>'])):
            sys.exit(1)

    with h5py.File(args['<source>'], 'r') as source:

        paths = verify(source, getdatasets(source))

        with h5py.File(args['<destination>'], 'w') as dest:
            copydatasets(source, dest, paths)



'''G5list
    List datasets (or groups of datasets) in a HDF5-file.

Usage:
    G5list [options] [--fold ARG]... <source>

Arguments:
    <source>    HDF5-file.

Options:
    -f, --fold=ARG        Fold paths.
    -d, --max-depth=ARG   Maximum depth to display.
    -r, --root=ARG        Start a certain point in the path-tree. [default: /]
    -i, --info            Print information: shape, dtype.
    -l, --long            As above but will all attributes.
    -h, --help            Show help.
        --version         Show version.

(c - MIT) T.W.J. de Geus | tom@geus.me | www.geus.me | github.com/tdegeus/GooseHDF5
'''

from .. import getpaths
from .. import __version__
import docopt
import h5py
import os
import warnings
warnings.filterwarnings("ignore")


def check_isfile(fname):
    if not os.path.isfile(fname):
        raise IOError('"{0:s}" does not exist'.format(fname))


def print_plain(source, paths):
    r'''
Print the paths to all datasets (one per line).
    '''

    for path in paths:
        print(path)


def print_info(source, paths):
    r'''
Print the paths to all datasets (one per line), including type information.
    '''

    def has_attributes(lst):
        for i in lst:
            if i != '-' and i != '0':
                return True
        return False

    out = {
        'path': [],
        'size': [],
        'shape': [],
        'dtype': [],
        'attrs': []}

    for path in paths:
        if path in source:
            data = source[path]
            out['path'] += [path]
            out['size'] += [str(data.size)]
            out['shape'] += [str(data.shape)]
            out['dtype'] += [str(data.dtype)]
            out['attrs'] += [str(len(data.attrs))]
        else:
            out['path'] += [path]
            out['size'] += ['-']
            out['shape'] += ['-']
            out['dtype'] += ['-']
            out['attrs'] += ['-']

    width = {}
    for key in out:
        width[key] = max([len(i) for i in out[key]])
        width[key] = max(width[key], len(key))

    fmt = '{0:%ds} {1:%ds} {2:%ds} {3:%ds}' % \
        (width['path'], width['size'], width['shape'], width['dtype'])

    if has_attributes(out['attrs']):
        fmt += ' {4:%ds}' % width['attrs']

    print(fmt.format('path', 'size', 'shape', 'dtype', 'attrs'))
    print(fmt.format(
            '=' * width['path'],
            '=' * width['size'],
            '=' * width['shape'],
            '=' * width['dtype'],
            '=' * width['attrs']))

    for i in range(len(out['path'])):
        print(fmt.format(
            out['path'][i],
            out['size'][i],
            out['shape'][i],
            out['dtype'][i],
            out['attrs'][i]))


def print_attribute(source, paths):

    for path in paths:
        if path in source:

            data = source[path]

            print('"{0:s}"'.format(path))
            print('- prop: size = {0:s}, shape = {1:s}, dtype = {2:s}'.format(
                str(data.size), str(data.shape), str(data.dtype)))

            for key in data.attrs:
                print('- attr: ' + key + ' = ')
                print('        ' + data.attrs[key])

            print('')

def main():
    r'''
Main function.
    '''

    args = docopt.docopt(__doc__, version=__version__)

    check_isfile(args['<source>'])

    with h5py.File(args['<source>'], 'r') as source:

        paths = getpaths(
            source,
            root=args['--root'],
            max_depth=args['--max-depth'],
            fold=args['--fold'])

        if args['--info']:
            print_info(source, paths)
        elif args['--long']:
            print_attribute(source, paths)
        else:
            print_plain(source, paths)



'''G5check
    Try reading datasets. In case of reading failure the path is printed (otherwise nothing is
    printed).

Usage:
    G5check <source> [options]

Arguments:
    <source>        HDF5-file.

Options:
    -b, --basic     Only try getting a list of datasets, skip trying to read them.
    -h, --help      Show help.
        --version   Show version.

(c - MIT) T.W.J. de Geus | tom@geus.me | www.geus.me | github.com/tdegeus/GooseHDF5
'''

from .. import verify
from .. import getdatasets
from .. import __version__
import docopt
import h5py
import os
import warnings
warnings.filterwarnings("ignore")


def check_isfile(fname):
    if not os.path.isfile(fname):
        raise IOError('"{0:s}" does not exist'.format(fname))


def read(filename, check):
    r'''
Read (and check) all datasets.
    '''

    with h5py.File(filename, 'r') as source:

        paths = getdatasets(source)

        if check:
            verify(source, paths, error=True)


def main():
    r'''
Main function.
    '''

    args = docopt.docopt(__doc__, version=__version__)

    check_isfile(args['<source>'])

    read(args['<source>'], not args['--basic'])

    try:
        read(args['<source>'], not args['--basic'])
    except:
        print(args['<source>'])




from setuptools import setup
from setuptools import find_packages

import re

filepath = 'GooseHDF5/__init__.py'
__version__ = re.findall(r'__version__ = \'(.*)\'', open(filepath).read())[0]

setup(
    name='GooseHDF5',
    version=__version__,
    license='MIT',
    author='Tom de Geus',
    author_email='tom@geus.me',
    description='Wrapper around h5py',
    long_description='Wrapper around h5py',
    keywords='HDF5, h5py',
    url='https://github.com/tdegeus/GooseHDF5',
    packages=find_packages(),
    install_requires=['docopt>=0.6.2', 'h5py>=2.8.0'],
    entry_points={
        'console_scripts': [
            'G5check = GooseHDF5.cli.G5check:main',
            'G5compare = GooseHDF5.cli.G5compare:main',
            'G5list = GooseHDF5.cli.G5list:main',
            'G5print = GooseHDF5.cli.G5print:main',
            'G5repack = GooseHDF5.cli.G5repack:main',
            'G5repair = GooseHDF5.cli.G5repair:main']})



'''G5repack
    Read and write a HDF5 file, to write it more efficiently by removing features like
    extendible datasets.

Usage:
    G5repack [options] <source>...

Arguments:
    <source>    HDF5-file.

Options:
    -c, --compress  Apply compression (using the loss-less GZip algorithm).
    -f, --float     Store doubles as floats.
    -h, --help      Show help.
        --version   Show version.

(c - MIT) T.W.J. de Geus | tom@geus.me | www.geus.me | github.com/tdegeus/GooseHDF5
'''

from .. import isnumeric
from .. import getpaths
from .. import copy_dataset
from .. import __version__
import docopt
import h5py
import os
import tempfile
import warnings
import numpy as np
warnings.filterwarnings("ignore")

def check_isfile(fname):
    if not os.path.isfile(fname):
        raise IOError('"{0:s}" does not exist'.format(fname))

def main():

    args = docopt.docopt(__doc__, version=__version__)
    tempname = next(tempfile._get_candidate_names())

    for filename in args['<source>']:

        print(filename)

        check_isfile(filename)

        with h5py.File(filename, 'r') as source:
            with h5py.File(tempname, 'w') as tmp:
                copy_dataset(source, tmp, getpaths(source), args['--compress'], args['--float'])

        os.replace(tempname, filename)



'''G5print
    Print datasets in a HDF5-file.

Usage:
    G5print [options] <source> [<dataset>...]

Arguments:
    <source>    HDF5-file.
    <dataset>   Path to the dataset.

Options:
    -r, --regex     Evaluate dataset name as a regular expression.
    -i, --info      Print information: shape, dtype.
    -a, --attrs     Print attributes.
        --no-data   Don't print data.
    -h, --help      Show help.
        --version   Show version.

(c - MIT) T.W.J. de Geus | tom@geus.me | www.geus.me | github.com/tdegeus/GooseHDF5
'''


from .. import getpaths
from .. import __version__
import docopt
import h5py
import re
import os
import sys
import warnings
warnings.filterwarnings("ignore")


def main():

    args = docopt.docopt(__doc__, version=__version__)

    if not os.path.isfile(args['<source>']):
        print('File does not exist')
        sys.exit(1)

    with h5py.File(args['<source>'], 'r') as source:

        if len(args['<dataset>']) == 0:
            print_header = True
            datasets = list(getpaths(source))
        elif args['--regex']:
            print_header = True
            paths = getpaths(source)
            datasets = []
            for dataset in args['<dataset>']:
                datasets += [path for path in paths if re.match(dataset, path)]
        else:
            datasets = args['<dataset>']
            print_header = len(datasets) > 1

        for dataset in datasets:
            if dataset not in source:
                print('"{0:s}" not in "{1:s}"'.format(dataset, source.filename))

        for i, dataset in enumerate(datasets):

            data = source[dataset]

            if args['--info']:
                print('path = {0:s}, size = {1:s}, shape = {2:s}, dtype = {3:s}'.format(
                    dataset,
                    str(data.size),
                    str(data.shape),
                    str(data.dtype),
                ))
            elif print_header:
                print(dataset)

            if args['--attrs']:
                for key in data.attrs:
                    print(key + ' : ' + str(data.attrs[key]))

            if not args['--no-data']:
                print(data[...])

            if len(datasets) > 1 and i < len(datasets) - 1:
                print('')
